/* pam_data.c */

#include <stdlib.h>
#include <string.h>

#include "pam_private.h"

struct pam_data *_pam_locate_data(const pam_handle_t *pamh, const char *name);

int pam_set_data(
    pam_handle_t *pamh,
    const char *module_data_name,
    const void *data,
    int (*cleanup)(pam_handle_t *pamh, const void *data, int error_status))
{
    struct pam_data *data_entry;
    
    IF_NO_PAMH("pam_set_data",pamh,PAM_SYSTEM_ERR);

    if ((data_entry = _pam_locate_data(pamh, module_data_name))) {
	/* Should cleanup() really be called here?
	 * According to SonOS 5.5 man page, no.
	 */
	if (data_entry->cleanup) {
	    data_entry->cleanup(pamh, data_entry->data, PAM_SUCCESS);
	}
	/* AGM uncommented the above since it plugs the memory leak
	 * here.  Actually, the Sun 5.5 manual (as of 1996/2/8) makes
	 * no mention of how this memory gets recovered and this
	 * really does seem to be the most natural means.
	 */
    } else if ((data_entry = malloc(sizeof(*data_entry)))) {
	char *tname;

	if ((tname = strdup(module_data_name)) == NULL) {
	    _pam_log_error("pam_set_data: no memory for data name");
	    DROP(data_entry);
	    return PAM_BUF_ERR;
	}
	data_entry->next = pamh->data;
	pamh->data = data_entry;
	data_entry->name = tname;
    } else {
	_pam_log_error("pam_set_data: cannot allocate data entry");
	return PAM_BUF_ERR;
    }

    data_entry->data = data;
    data_entry->cleanup = cleanup;

    return PAM_SUCCESS;
}

int pam_get_data(
    const pam_handle_t *pamh,
    const char *module_data_name,
    const void **datap)
{
    struct pam_data *data;

    IF_NO_PAMH("pam_get_data",pamh,PAM_SYSTEM_ERR);

    data = _pam_locate_data(pamh, module_data_name);
    if (data) {
	*datap = data->data;
	return PAM_SUCCESS;
    }

    return PAM_NO_MODULE_DATA;
}

struct pam_data *_pam_locate_data(const pam_handle_t *pamh, const char *name)
{
    struct pam_data *data;

    IF_NO_PAMH("_pam_locate_data",pamh,NULL);
    data = pamh->data;
    
    while (data) {
	if (!strcmp(data->name, name)) {
	    return data;
	}
	data = data->next;
    }

    return NULL;
}


void _pam_free_data(pam_handle_t *pamh, int status)
{
    struct pam_data *last;
    struct pam_data *data;

    IF_NO_PAMH("_pam_free_data",pamh,/* no return value for void fn */);
    data = pamh->data;

    while (data) {
	last = data;
	data = data->next;
	if (last->cleanup) {
	    last->cleanup(pamh, last->data, status);
	}
	DROP(last->name);
	DROP(last);
    }
}
